c=======================================================================
c
c     URRFIT (2015-1) *Extrapolate cross section moments from RRR to URR
c                      to define self-shielding and 2 band parameters.
c            (2016-1) *For a bands conserve 1/[SigmaT + sigma0], where
c                      sigma0 = the unshielded average total
c            (2016-2) *Updated multi-band treatment to explicitly handle
c                      small shielding limit - without this update the
c                      small limit becomes numerically unstable.
c                     *Increased maximum URR energies from 100 to 500.
c
c     WARNING - This only does the first MAT on the MULTIBAND Input File
c     ==================================================================
c
c     I/O File Definitions
c     ====================
c       2   URRFIT.INP
c       3   URRFIT.LST
c      10   GROUPIE Output Listing (variable input - see below)
c      12   Output for TART        (variable input - see below)
c      18   Output for NJOY        (variable input - see below)
c      14   NJOY.CUR      = F1 and F2 vs. Energy - Output by URRDO
c      16   VS-ENERGY.CUR = F1 and F2 vs. Energy
c      17   VS-SIGMA0.CUR = F1 vs. Sigma0 at first and last energies
c
c     Input Parameters (in URRFIT.INP)
c     ================================
c     Line  Columns Definition
c     ========================
c        1  1-11    ZA
c          12-22    MAT
c          23-33    Atomic Weight Ratio
c          34-44    LOwer Energy Limit of Unresolved Resonance Region
c          45-55    Upper Energy Limit of Unresolved Resonance Region
c          56-66    Competitive Channel MT
c          67-70    LSSF = Unresolved Tabulated Flag (0 or 1)
c        2  1-72    GROUPIE MULTBAND.TAB Output Created by GROUPIE
c        3  1-72                 "              Created by this code
c        4  1-72    NJOY MF/MT=2/152 and 2/153  Created by this code
c
c     Method
c     ======
c     This code reads the GROUPIE ouput file MULTBAND.TAB that defines
c     2 band parameters in the resolved energy range. These are used
c     to define F1 and F2 self-shielding factors vs. Energy. These
c     F1 and F2 are then extrapolated into the unresolved resonance
c     region. The extrapolatd F1 and F2 values are then used to derive
c     2 band parameters in the unresolved resonance region.
c
c     Results are output,
c     1) For use by TART in the same MULTBAND.TAB format as read
c     2) For use by NJOY in ENDF format MF/MT=2/152 and 2/153
c
c     This procedure avoids the approximations used by other methods
c     and guarantees a smooth, physically variation of the f-factors
c     across the entire resolved and unresolved energy range.
c
c     Some of the approximations used by other methods, but not here,
c     include restriction to single level Breit-Wigner resonances in
c     the unresolved resonance region, and psi-chi method to define
c     Doppler brfoadening - these are not appropriate to accurately
c     define minima between resonance (according to Eugene Wigner, who
c     should know something about the Breit-Wigner resonance formalism).
c
c     Also other methods use random sampling of ladders of resonances,
c     which is very time consuming introducing statistical uncertainty
c     in the answer.
c
c     Here by focussing on macroscopic physical observable moments of
c     the cross section, whatever formalism is used in the resolved
c     resonance energy range is smoothly extrapolated into the
c     unresolved resonance energy range with no statistical uncertainty.
c
c=======================================================================
c
c     Definition of Self-Shielding Factors
c     ====================================
c
c     Bonderenko self-shielded cross section is defined as,
c
c                   integral[sig(E)*dE/{sigT(E)+sigma0}]
c     sig(sigma0) = ------------------------------------
c                   integral[dE/{sigT(E)+sigma0}]
c
c     In terms of discrete multi-band or PTM parameters the integrals
c     are replaced by weighted sums.
c
c                   sum[wtB*sigB/{sigTB+sigma0}]
c     sig(sigma0) = -------------------------------
c                   sum[wtB/{sigTB+sigma0}]
c
c     Below I define F0 for sigma0=infinity
c                and F1 for sigma0=0
c                and F2 using SigTB**2 instead of SigTB,
c
c      F0         = sum[wtB*sigB]            [Unshielded]
c
c                   sum[wtB*sigB/sigTB]
c      F1         = ----------------------   [Flux weigthing]
c                   sum[wtB/sigTB]
c
c                   sum[wtB*sigB/sigTB]
c      F2         = ----------------------   [Current weighting]
c                   sum[wtB/sigTB**2]
c
c     In terms of f-factors the cross sections are replaced by the
c     f-factors and in all cases the results scale by the unshielded
c     average cross section for each reaction,
c
c      F0         = sum[wtB*f-factor] - should = 1.
c
c                   sum[wtB*f-factorTB/f-factorTB]
c      F1         = ------------------------------
c                   sum[wtB/f-factorTB]
c
c                   sum[wtB*f-factorTB/f-factorTB**2]
c      F2         = ---------------------------------
c                   sum[wtB/f-factorTB**2]
c
c=======================================================================
      INCLUDE 'implicit.h'
      character*8 codename
      character*72 filename
      COMMON/HEADER/C1H,C2H,L1H,L2H,N1H,N2H,MATH,MFH,MTH,NOSEQ
      common/leader/c1 , c2, l1, l2, n1, n2
      common/fitcom/egroup(1000),eav(1000),xcav(4,1000)
      common/band2com/wt(2,1000),xmb(2,4,1000)
      common/mommy/xcmom(0:2,4,1000),xcnew(0:2,4,1000)
      common/mbparamf/tempk,iza,ngroup,nband
      common/mbparamc/title,zahol
      common/sig0com/AE1(4),AE2(4),eurr(1000),neurr
      common/njoycom/njoycur,nunr,LSSF,INTUNR,ICOMP
      common/namecode/codename
      common/urrlimits/elow,ehigh
      common/BMETHOD/METHODB
      INTEGER*4 OUTP,OTAPE
      COMMON/ENDFIO/INP,OUTP,ITAPE,OTAPE
      character*12 zahol
      character*72 title
      character*1 field3(11,3)
c-----Define codename for compatibility with PREPRO
      codename = 'URRFIT  '
c-----Method = Conserve 1/[Total + <Total>]
      METHODB = 2
c-----------------------------------------------------------------------
c
c     Define I/O units and all input prameters.
c
c-----------------------------------------------------------------------
      open(3,file='URRFIT.LST')
      write(3,10)
      write(*,10)
   10 format(///' URRFIT (2016-2):',
     1          ' Unresolved Shielding Factors from GROUPIE Output'/
     1 1x,71('='))
      open(2,file='URRFIT.INP',status='OLD',err=20)
      go to 40
   20 write(*,30)
   30 format(///' ERROR - Cannot find URRFIT.INP.'/
     1          '         Execution terminated.'///)
      call enderror
   40 read(2,50) izain,MATH,atwt,elow,ehigh,ICOMP,LSSF
      INTUNR = 5
   50 format(2i11,3d11.4,i11,i4)
      call out9g(atwt ,field3(1,1))
      call out9g(elow ,field3(1,2))
      call out9g(ehigh,field3(1,3))
      if(ICOMP.lt.0) ICOMP = 0
      if(LSSF .ne.0)  LSSF = 1
      write(3,60) izain,MATH,field3,ICOMP,LSSF
      write(*,60) izain,MATH,field3,ICOMP,LSSF
   60 format(
     1 ' Material (ZA)...................',i11/
     1 ' MAT.............................',i11/
     2 ' Atomic Weight...................',11A1/
     2 ' Unresolved Lower Energy Limit...',11A1,' eV'/
     3 ' Unresolved Upper Energy Limit...',11A1,' eV'/
     4 ' Competitive Channel (MT)........',I11/
     5 ' Unresolved Tabulated?(LSSF).....',I11/
     6 1x,71('-'))
      if(MATH.lt.1.or.MATH.gt.9999) then
      write(3,70)
   70 format(///' ERROR - MAT MUST be 1 to 9999.'/
     1          '         Execution Terminated.'///)
      call enderror
      endif
c-----------------------------------------------------------------------
c
c     Define I/O units and filenames
c
c-----------------------------------------------------------------------
c-----MULTBAND.TAB = Groupie Output Listing of Multi-band parameters
      read(2,80) filename
   80 format(A72)
      write(3,90) filename
      write(*,90) filename
   90 format(' GROUPIE Output.....',A72)
      open(10,file=filename,status='OLD',err=100)
      go to 120
  100 write(*,110)
  110 format(///' ERROR - Cannot find ',A72/
     1          '         Execution terminated.'///)
      call enderror
c-----MULTBAND.NEW = Updated to include URR self-shielding
  120 read(2,80) filename
      write(3,130) filename
      write(*,130) filename
  130 format(' Output for TART....',A72)
      open(12,file=filename,status='UNKNOWN')
c-----NJOY.MF2 = Multi-band parameters for NJOY
      read(2,80) filename
      write(3,140) filename
      write(*,140) filename
  140 format(' Output for NJOY....',A72)
      open(18,file=filename,status='UNKNOWN')
c-----Optional NJOY results in PLOTTAB format.
      open(14,file='NJOY.CUR',status='OLD',err=150)
      njoycur = 1
      go to 160
  150 njoycur = 0
c-----PLOTTAB output.
  160 open(16,file='VS-ENERGY.CUR',status='UNKNOWN')
      open(17,file='VS-SIGMA0.CUR',status='UNKNOWN')
      OTAPE = 18
      write(3,170)
      write(*,170)
  170 format(1x,71('='))
c-----------------------------------------------------------------------
c
c     Define Unresolved Energy Region.
c
c-----------------------------------------------------------------------
      write(18,180)
  180 format(' URRFIT(2016-2) NJOY MB MF/MT=2/152 and 2/153',21x,
     1 '7000 0  0    0')
c             123456789012345678901234567890123456789012345
      neurr = 2
      eurr(1) = elow
      eurr(2) = ehigh
c-----------------------------------------------------------------------
c
c     Read entire table of multiband parameters.
c
c-----------------------------------------------------------------------
c-----define parameters
      read(10,190) title
  190 format(a72)
      read(10,200) iza,ngroup,nband,tempk,zahol
  200 format(3i11,d11.4,a12)
c-----Check ZA
      if(iza.ne.izain) then
      write(3,210) iza
      write(*,210) iza
  210 format(///
     1 ' ERROR - Wrong ZA GROUPIE Output=',i11/
     2 '         Execution Terminated.'///)
      call enderror
      endif
c-----Check ngroup = only allow 616
      if(ngroup.ne.616) then
      write(3,220) ngroup
      write(*,220) ngroup
  220 format(///
     1 ' ERROR - Wrong number of GROUPIE output groups=',i11/
     2 '         URRFIT is designd to create input for both'/
     3 '         TART and MCNP and the methods used are'/
     4 '         only appropriate for the TART 616 Group'/
     5 '         structure = -11 input to GROUPIE.'///)
      call enderror
      endif
c-----Check nband = only allow 2
      if(nband.ne.2) then
      write(3,230) nband
      write(*,230) nband
  230 format(///
     1 ' ERROR - Wrong number of GROUPIE output bands=',i11/
     2 '         URRFIT is designd to create input for both'/
     3 '         TART and MCNP and the methods used are'/
     4 '         only appropriate for the 2 cross section'/
     5 '         bands.'///)
      call enderror
      endif
c-----define multband data
      do igroup=1,ngroup
      read(10,240) egroup(igroup),wt(1,igroup),(xmb(1,k,igroup),k=1,4)
  240 format(6d11.4)
      read(10,250)                wt(2,igroup),(xmb(2,k,igroup),k=1,4)
  250 format(11x,5d11.4)
      enddo
      read(10,240) egroup(ngroup+1)
c-----------------------------------------------------------------------
c
c     Define NJOY ENDF format output parameters,
c
c-----------------------------------------------------------------------
      C1H = iza              ! HEADER line
      C2H = atwt
      L1H = 0
      L2H = 0
      N1H = 0
      N2H = 0
      c1  = tempk            ! LEADER line
      c2  = 0.0d0
      l1  = 0
      l2  = 0
      n1  = 0
      n2  = 0
c-----------------------------------------------------------------------
c
c     Define group average energy and cross section
c
c-----------------------------------------------------------------------
      do igroup=1,ngroup
      eav (igroup) = 0.5*(egroup(igroup) + egroup(igroup+1))
      do k=1,4
      xcav(k,igroup) =  wt(1,igroup)*xmb(1,k,igroup) +
     1                  wt(2,igroup)*xmb(2,k,igroup)
      enddo
      enddo
c-----------------------------------------------------------------------
c
c     calculate moments from multband parameters
c
c-----------------------------------------------------------------------
      call moments
c-----------------------------------------------------------------------
c
c     fit amount of URR shieldng: d = A/E
c
c-----------------------------------------------------------------------
      call fitz(elow,ehigh)
c-----------------------------------------------------------------------
c
c     define new multiband parameters
c
c-----------------------------------------------------------------------
      call mbmaker(elow,ehigh)
c-----------------------------------------------------------------------
c
c     output updated multiband parameters
c
c-----------------------------------------------------------------------
      call newmb        ! For TART
      call mfmt2152     ! For NJOY
      call mfmt2153
c-----------------------------------------------------------------------
c
c     PLOTTAB output = fixed energies vs. SIGMA-0
c
c-----------------------------------------------------------------------
      call vssigma0
c-----------------------------------------------------------------------
c
c     PLOTTAB output - f0, f1, f2 vs. ENERGY
c
c-----------------------------------------------------------------------
      call plottab
c-----------------------------------------------------------------------
c
c     end of run
c
c-----------------------------------------------------------------------
c-----Add FEND/MEND/TEND to end of Output for NJOY
      write(18,260) MATH
  260 format(66x, i4,' 0  0    0'/
     1       66x,'   0 0  0    0'/
     2       66x,'  -1 0  0    0')
      call endit
      end
      subroutine moments
c=======================================================================
c
c     Define moments starting from multiband parameters.
c
c=======================================================================
      INCLUDE 'implicit.h'
      common/fitcom/egroup(1000),eav(1000),xcav(4,1000)
      common/band2com/wt(2,1000),xmb(2,4,1000)
      common/mommy/xcmom(0:2,4,1000),xcnew(0:2,4,1000)
      common/mbparamf/tempk,iza,ngroup,nband
      common/BMETHOD/METHODB
c-----------------------------------------------------------------------
c
c     Loop over groups
c
c-----------------------------------------------------------------------
      do igroup=1,ngroup
c-----denominators = same for all 4 reactions
      sum0t = wt(1,igroup)                   +
     1        wt(2,igroup)
      sum1t = wt(1,igroup)/xmb(1,1,igroup)    +
     1        wt(2,igroup)/xmb(2,1,igroup)
      if(METHODB.eq.1) then
      sub1  = xmb(1,1,igroup)**2           ! 1/S^2
      sub2  = xmb(2,1,igroup)**2
      else
      sigma0= wt(1,igroup)*xmb(1,1,igroup) +
     1        wt(2,igroup)*xmb(2,1,igroup)
      sub1  = xmb(1,1,igroup) + sigma0     ! 1/[s+<S>]
      sub2  = xmb(2,1,igroup) + sigma0
      endif
      sum2t = wt(1,igroup)/sub1 +
     1        wt(2,igroup)/sub2
c-----------------------------------------------------------------------
c
c     Total, Elastic, Capture, Fission
c
c-----------------------------------------------------------------------
      do ireact=1,4
c-----numerator = different for each reaction.
      sum0r = wt(1,igroup)*xmb(1,ireact,igroup)      +
     1        wt(2,igroup)*xmb(2,ireact,igroup)
      sum1r = wt(1,igroup)*xmb(1,ireact,igroup)/xmb(1,1,igroup) +
     1        wt(2,igroup)*xmb(2,ireact,igroup)/xmb(2,1,igroup)
c-----No METHODB Test = Now the same = sub1 and sub2 correct
      sum2r = wt(1,igroup)*xmb(1,ireact,igroup)/sub1 +
     1        wt(2,igroup)*xmb(2,ireact,igroup)/sub2
c-----Moments
      xcmom(0,ireact,igroup) = sum0r/sum0t
      xcmom(1,ireact,igroup) = sum1r/sum1t
      xcmom(2,ireact,igroup) = sum2r/sum2t
c-----F factors
      if(xcmom(0,ireact,igroup).gt.0.0) then
      xcmom(2,ireact,igroup) = xcmom(2,ireact,igroup)/
     1                         xcmom(0,ireact,igroup)
      xcmom(1,ireact,igroup) = xcmom(1,ireact,igroup)/
     1                         xcmom(0,ireact,igroup)
      xcmom(0,ireact,igroup) = xcmom(0,ireact,igroup)/
     1                         xcmom(0,ireact,igroup)
      else
      xcmom(2,ireact,igroup) = 1.0   ! usually no fission
      xcmom(1,ireact,igroup) = 1.0
      xcmom(0,ireact,igroup) = 1.0
      endif
c-----initialize to be same
      xcnew(2,ireact,igroup) = xcmom(2,ireact,igroup)
      xcnew(1,ireact,igroup) = xcmom(1,ireact,igroup)
      xcnew(0,ireact,igroup) = xcmom(0,ireact,igroup)
      enddo
      enddo
      return
      end
      subroutine fitz(elow,ehigh)
c=======================================================================
c
c     PLOTTAB formatted output.
c
c=======================================================================
      INCLUDE 'implicit.h'
      common/fitcom/egroup(1000),eav(1000),xcav(4,1000)
      common/band2com/wt(2,1000),xmb(2,4,1000)
      common/mommy/xcmom(0:2,4,1000),xcnew(0:2,4,1000)
      common/mbparamf/tempk,iza,ngroup,nband
      common/sig0com/AE1(4),AE2(4),eurr(1000),neurr
      character*12 what(4)
      data what/
     1 'Total       ',
     2 'Elastic     ',
     3 'Capture     ',
     4 'Fission     '/
c-----fit is below URR
      e1 = 0.5 *elow
      e2 = 0.9 *elow
c-----------------------------------------------------------------------
c
c     Loop over reactions
c
c-----------------------------------------------------------------------
      do ireact=1,4
c-----Initialize fit parameters.
      a1 = 0.0
      b1 = 0.0
      a2 = 0.0
      b2 = 0.0
c-----groups
      do 10 igroup=1,ngroup
      if(eav(igroup).lt.e1) go to 10
      if(eav(igroup).gt.e2) go to 20
      d1 = 1.0 - xcmom(1,ireact,igroup)
      a1 = a1 + eav(igroup)*d1
      b1 = b1 + 1.0
      d2 = 1.0 - xcmom(2,ireact,igroup)
      a2 = a2 + eav(igroup)*d2
      b2 = b2 + 1.0
   10 continue
c-----------------------------------------------------------------------
c
c     define fit
c
c-----------------------------------------------------------------------
c-----Test for no points in range = set to NO SHIELDING.
   20 if(b1.le.0.0d0.or.b2.le.0.0d0) then
      AE1(ireact) = 0.0d0
      AE2(ireact) = 0.0d0
      else
      AE1(ireact) = a1/b1
      AE2(ireact) = a2/b2
      endif
c-----f0 = 1
      do 30 igroup=1,ngroup
      if(eav(igroup).lt.e1) go to 30
      if(eav(igroup).gt.1.5*ehigh) go to 40
      f0fit = 1.0
      xcnew(0,ireact,igroup) = f0fit
   30 continue
c-----f1
   40 do 50 igroup=1,ngroup
      if(eav(igroup).lt.e1) go to 50
      if(eav(igroup).gt.1.5*ehigh) go to 60
      dd = AE1(ireact)/eav(igroup)
      f1fit = 1.0 - dd
      xcnew(1,ireact,igroup) = f1fit
   50 continue
c-----f2
   60 do 70 igroup=1,ngroup
      if(eav(igroup).lt.e1) go to 70
      if(eav(igroup).gt.1.5*ehigh) go to 80
      dd = AE2(ireact)/eav(igroup)
      f2fit = 1.0 - dd
      xcnew(2,ireact,igroup) = f2fit
   70 continue
   80 continue
      enddo
      return
      end
      subroutine mbmaker(elow,ehigh)
c=======================================================================
c
c     Create Multi-Band Parameters
c
c     Verified vs. GROUPIE and Output here.
c
c     Convert from f-factors to barns
c
c=======================================================================
      INCLUDE 'implicit.h'
      INCLUDE 'urrfit.h'
      common/fitcom/egroup(1000),eav(1000),xcav(4,1000)
      common/band2com/wt(2,1000),xmb(2,4,1000)
      common/mommy/xcmom(0:2,4,1000),xcnew(0:2,4,1000)
      common/mbparamf/tempk,iza,ngroup,nband
      common/njoycom/njoycur,nunr,LSSF,INTUNR,ICOMP
c-----------------------------------------------------------------------
c
c     Replace multiband parameters through URR
c
c-----------------------------------------------------------------------
c-----all of URR
      e1 =      elow
      e2 =      ehigh
c-----Initialize NJOY group count
      nunr = 0
      do 10 igroup=1,ngroup
c-----select if any part of group is within unresolved.
      if(egroup(igroup+1).le.e1) go to 10
      if(egroup(igroup)  .ge.e2) go to 20
c-----next group.
      call mbmake1(xcnew(0,1,igroup),wt(1,igroup),xmb(1,1,igroup))
      nunr = nunr + 1
      eunr(nunr)   = eav(igroup)
c-----------------------------------------------------------------------
c
c     Data for NJOY
c
c     Note, exchange F and C order from PREPRO to NJOY Convention
c
c-----------------------------------------------------------------------
c
c     MF/MT=2/153 probability output (switch C and F)
c      6 = Wt T E F C Heat = NJOY order
c
c-----LSSF = 0 - Output Cross Sections
      if(LSSF.eq.0) then
      do ib=1,2
      bands(ib,1,nunr) = wt( ib,igroup)                  ! wt
      bands(ib,2,nunr) = xmb(ib,1,igroup)*xcav(1,igroup) ! T
      bands(ib,3,nunr) = xmb(ib,2,igroup)*xcav(2,igroup) ! E
      bands(ib,4,nunr) = xmb(ib,4,igroup)*xcav(4,igroup) ! F
      bands(ib,5,nunr) = xmb(ib,3,igroup)*xcav(3,igroup) ! C
      bands(ib,6,nunr) = 0.0d0                           ! heat
      enddo
      else
c-----LSSF = 1 - Output f-factors
      do ib=1,2
      bands(ib,1,nunr) = wt( ib,igroup)     ! wt
      bands(ib,2,nunr) = xmb(ib,1,igroup)   ! T
      bands(ib,3,nunr) = xmb(ib,2,igroup)   ! E
      bands(ib,4,nunr) = xmb(ib,4,igroup)   ! F
      bands(ib,5,nunr) = xmb(ib,3,igroup)   ! C
      bands(ib,6,nunr) = 0.0d0              ! heat
      enddo
      endif
c
c     MF/MT=2/152 shielded cross section output
c     unshieded
c
      sigu(1,1,nunr) = xcav(1,igroup)       ! T
      sigu(2,1,nunr) = xcav(2,igroup)       ! E
      sigu(3,1,nunr) = xcav(4,igroup)       ! F
      sigu(4,1,nunr) = xcav(3,igroup)       ! C
c-----totally shielded
      bottom = wt(1,igroup)/bands(1,2,nunr) +
     1         wt(2,igroup)/bands(2,2,nunr)
      do ir=1,4
      top    = wt(1,igroup)*bands(1,ir+1,nunr)/bands(1,2,nunr) +
     1         wt(2,igroup)*bands(2,ir+1,nunr)/bands(2,2,nunr)
      sigu(ir,2,nunr) = sigu(ir,1,nunr)*top/bottom
      enddo
c-----Current
      bottom = wt(1,igroup)/bands(1,2,nunr)**2 +
     1         wt(2,igroup)/bands(2,2,nunr)**2
      top    = wt(1,igroup)*bands(1,2,nunr)/bands(1,2,nunr)**2 +
     1         wt(2,igroup)*bands(2,2,nunr)/bands(2,2,nunr)**2
      sigu(5,1,nunr) = sigu(1,1,nunr)
      sigu(5,2,nunr) = sigu(1,1,nunr)*top/bottom
c-----------------------------------------------------------------------
c
c     Data for TART
c
c-----------------------------------------------------------------------
c-----Convert from f-factors to barns for output
      do ireact=1,4
      xmb(1,ireact,igroup) = xmb(1,ireact,igroup)*xcav(ireact,igroup)
      xmb(2,ireact,igroup) = xmb(2,ireact,igroup)*xcav(ireact,igroup)
      enddo
c-----End of Group loop
   10 continue
c-----End of URR calculation
   20 return
      end
      subroutine mbmake1(xcnew1,wt1,xmb1)
c=======================================================================
c
c     Create Multi-Band Parameters for 1 group
c
c     Verified vs. GROUPIE and Output here.
c
c=======================================================================
      INCLUDE 'implicit.h'
      common/BMETHOD/METHODB
      dimension xcnew1(0:2,4),wt1(2),xmb1(2,4)
c-----------------------------------------------------------------------
c
c     Method #1: Total Parameters.
c
c     Conserve F0, F1 and F2 = 1/total**2
c
c     SIG1X = SIG1T + SIGMA0
c     SIG2X = SIG2T + SIGMA0
c F0 = P1*SIG1T         + P2*SIG2T
c F1 =(P1*SIG1T/SIG1T   + P2*SIG2T/SIG2T  )/(P1/SIG1T   + P2/SIG2T  )
c F2 =(P1*SIG1T/SIG1X   + P2*SIG2T/SIG2X  )/(P1/SIG1X   + P2/SIG2X  )
c
c-----------------------------------------------------------------------
c
c     Method #2: Total Parameters.
c
c     Conserve F0, F1 and F2 = 1/(total + <total>)
c
c F0 = P1*SIG1T         + P2*SIG2T
c F1 =(P1*SIG1T/SIG1T   + P2*SIG2T/SIG2T  )/(P1/SIG1T   + P2/SIG2T  )
c F2 =(P1*SIG1T/SIG1T^2 + P2*SIG2T/SIG2T^2)/(P1/SIG1T^2 + P2/SIG2T^2)
c
c-----------------------------------------------------------------------
      F0 = xcnew1(0,1)
      F1 = xcnew1(1,1)
      F2 = xcnew1(2,1)
      IMSMALL = 0
c-----------------------------------------------------------------------
c
c     NO or Little Self-Shielding Limits
c     NO     = use unshielded
c     Little = conserve F0 and F1
c
c-----------------------------------------------------------------------
      if(F0.le.0.0d0.or.F1.le.0.0d0.or.F2.le.0.0d0) go to 10
c-----no shielding limit.
      if(F1.ge.F0) go to 10
      if(F1.le.0.9999*F0) go to 20
c-----small shielding limit.
      IMSMALL = 1
      A       = 1.0/F1
      BB2     = (F0 - F1)/F0
      if(BB2.le.0.0d0) go to 10
      B       = DSQRT(BB2)/F1
      D       = 0.0d0
      go to 30
c-----------------------------------------------------------------------
c
c     Little or no self-shielding = assume none.
c
c-----------------------------------------------------------------------
   10 wt1(1) = 0.5d0
      wt1(2) = 0.5d0
      do k=1,4
      xmb1(1,k) = xcnew1(0,1)
      xmb1(2,k) = xcnew1(0,1)
      enddo
      return
c-----------------------------------------------------------------------
c
c     Normal Case - Insure ALL parameters are physically possible =
c     cross sections and weights are ALL positive.
c
c-----------------------------------------------------------------------
   20 top    = F0 - F2
      bottom = F0 - F1
      if(METHODB.eq.1) then
      A      = 0.5d0*top/(F2*bottom)
      else
      A      = 0.5d0*bottom/(F1*top)
      endif
      BB2    = (F1*A*(F0*A-2.0d0)+1.0d0)/(F0*F1) ! GROUPIE =O.K.
      if(BB2.le.0.0d0) go to 10
      B      = dsqrt(BB2)
c
c     Insure results are physically acceptable =
c     Weights and Cross Sections MUST BE POSITIVE.
c
      if(B.gt.0.9999*A) B = 0.9999*A       ! BIG self-shielding
      D      = (1.0d0 - A*F1)/(2.0d0*B*F1)
      if(D.gt. 0.4999) D = 0.4999          ! BIG self-shielding
      if(D.lt.-0.4999) D =-0.4999
c
c     Define final results.
c
   30 P1     = 0.5d0 + D
      P2     = 0.5d0 - D
      SIG1T  = 1.0d0/(A+B)
      SIG2T  = 1.0d0/(A-B)
c
c     Insure EXACT unshoelded: F0 = P1*SIG1T + P2*SIG2T
c     P1 + P2 = 1 = guaranteed from above definition,
c     USE SIG1T and SIG2T to EXACTLY re-normalize.
c
      F0X = P1*SIG1T + P2*SIG2T
      F0N = F0/F0X
      SIG1T = F0N*SIG1T
      SIG2T = F0N*SIG2T
c-----------------------------------------------------------------------
c
c     Save Total Parameters
c
c-----------------------------------------------------------------------
      wt1(1) = P1
      wt1(2) = P2
      xmb1(1,1) = SIG1T
      xmb1(2,1) = SIG2T
c-----------------------------------------------------------------------
c
c     Partial Parameters.
c
c     Conserve F0R and F1R - Insure cross sections are positive.
c
c     F0R = P1*SIG1R       + P2*SIG2R
c     F1R =(P1*SIG1R/SIG1T + P2*SIG2R/SIG2T)/(P1/SIG1T+P2/SIG2T)
c
c-----------------------------------------------------------------------
      BOTTOM1 = P1/SIG1T    + P2/SIG2T
c-----No METHODB test necessary - SIG1X and SIG2X are correct.
      BOTTOM2 = P1/SIG1X    + P2/SIG2X
      DIFF    =(1.0d0/SIG1T - 1.0d0/SIG2T)*F1
      do k=2,4
      F0R = xcnew1(0,k)
      F1R = xcnew1(1,k)
      F2R = xcnew1(2,k)
c-----------------------------------------------------------------------
c
c     No Negative Cross Sections.
c
c-----------------------------------------------------------------------
      if(F0R.le.0.0) then
      F0R = 0.0d0
      F1R = 0.0d0
      F2R = 0.0d0
      endif
c-----------------------------------------------------------------------
c
c     Any value of C will conserve unshielded average.
c     Try to also conserve totally shielded average,
c     but limit C to physical range for cross sections > 0
c
c-----------------------------------------------------------------------
      if(DIFF.eq.0.0d0) then
      SIG1R = F0R            ! No self-shielding
      SIG2R = F0R
      else
      C = (F0R - F1R)/DIFF
      if(C.gt.0.0d0) then
      if(C.gt. 0.999*P1*F0R) C = 0.999*P1*F0R ! BIG self-shielding
      else
      if(C.lt.-0.999*P2*F0R) C =-0.999*P2*F0R ! BIG self-shielding
      endif
      SIG1R = F0R - C/P1 ! With above constraints these MUST be > 0
      SIG2R = F0R + C/P2
      endif
c-----------------------------------------------------------------------
c
c     Save Results
c
c-----------------------------------------------------------------------
      xmb1(1,k) = SIG1R
      xmb1(2,k) = SIG2R
      enddo
      return
      end
      subroutine newmb
c=======================================================================
c
c     output updated multiband parameters for TART
c
c=======================================================================
      INCLUDE 'implicit.h'
      common/fitcom/egroup(1000),eav(1000),xcav(4,1000)
      common/band2com/wt(2,1000),xmb(2,4,1000)
      common/mommy/xcmom(0:2,4,1000),xcnew(0:2,4,1000)
      common/mbparamf/tempk,iza,ngroup,nband
      common/mbparamc/title,zahol
      character*12 zahol
      character*72 title
      character*1 fields(11,6)
c-----define parameters
      write(12,10) title
   10 format(a72)
      call out9g(tempk,fields(1,1))
      write(12,20) iza,ngroup,nband,(fields(k,1),k=1,11),zahol
   20 format(3i11,11a1,a12)
c-----define multband data
      do igroup=1,ngroup
      call out9g(egroup(igroup)  ,fields(1,1))
      call out9g(wt    (1,igroup),fields(1,2))
      do k=1,4
      call out9g(xmb(1,k,igroup)  ,fields(1,k+2))
      enddo
      write(12,30) ((fields(k,i),k=1,11),i=1,6)
   30 format(66a1)
      call out9g(wt    (2,igroup),fields(1,2))
      do k=1,4
      call out9g(xmb(2,k,igroup)  ,fields(1,k+2))
      enddo
      write(12,40) ((fields(k,i),k=1,11),i=2,6)
   40 format(11x,55a1)
      enddo
      call out9g(egroup(ngroup+1),fields(1,1))
      write(12,30) (fields(k,1),k=1,11)
      return
      end
      subroutine vssigma0
c=======================================================================
c
c     PLOTTAB output of self-shielding vs. sigma1 at fixed E
c
c=======================================================================
      INCLUDE 'implicit.h'
      common/sig0com/AE1(4),AE2(4),eurr(1000),neurr
      dimension xcx(0:2,4),wtx(2),xmbx(2,4),sig0(41)
      character*12 what(4)
      data what/
     1 'Total       ',
     2 'Elastic     ',
     3 'Capture     ',
     4 'Fission     '/
c-----nothing to do if no fixed energies
      if(neurr.le.0) return
c-----sigma0 as multiple of unshielded
      xx = 1.0/1024.0d0
      dx = dsqrt(2.0d0)
      do i=1,41
      sig0(i) = xx
      xx      = dx*xx
      enddo
c-----loop over fixed E
      do ieurr=1,neurr
      do ireact=1,4           ! define all moments
      xcx(0,ireact) = 1.0
      xcx(1,ireact) = 1.0 - AE1(ireact)/eurr(ieurr)
      xcx(2,ireact) = 1.0 - AE2(ireact)/eurr(ieurr)
      enddo
      call mbmake1(xcx,wtx,xmbx) ! define 2 band parameters
c-----curve vs. sigma0
      do ireact=1,4
      write(17,10) eurr(ieurr),what(ireact)
   10 format(1pd11.4,' eV ',a12)
      do is=1,41
      top = wtx(1)*xmbx(1,ireact)/(sig0(is)+xmbx(1,1)) +
     1      wtx(2)*xmbx(2,ireact)/(sig0(is)+xmbx(2,1))
      bot = wtx(1)               /(sig0(is)+xmbx(1,1)) +
     1      wtx(2)               /(sig0(is)+xmbx(2,1))
      yy  = top/bot
      write(17,20) sig0(is),yy
   20 format(1p2d11.4)
      enddo
      write(17,30)
   30 format(30x,'(blank line)')
      enddo
      enddo
      return
      end
      subroutine plottab
c=======================================================================
c
c     PLOTTAB VS-ENERGY.
c
c     Use NJOY order
c     1) Total
c     2) Elastic
c     3) Fission
c     4) Capture
c     Interchange 3 and 4
c
c=======================================================================
      INCLUDE 'implicit.h'
      common/fitcom/egroup(1000),eav(1000),xcav(4,1000)
      common/band2com/wt(2,1000),xmb(2,4,1000)
      common/mommy/xcmom(0:2,4,1000),xcnew(0:2,4,1000)
      common/mbparamf/tempk,iza,ngroup,nband
      common/urrlimits/elow,ehigh
      common/njoycom/njoycur,nunr,LSSF,INTUNR,ICOMP
      character*12 what(4)
      dimension myorder(4)
      character*60 line60
      character*22 line22
      equivalence (line60,line22)
      data myorder/1,2,4,3/
      data what/
     1 'Total       ',
     2 'Elastic     ',
     3 'Capture     ',
     4 'Fission     '/
c       123456789012
c-----------------------------------------------------------------------
c
c     Loop over reactions
c
c-----------------------------------------------------------------------
      e1 = 0.5*elow
      e2 = 1.2*ehigh
      do kreact=1,4
      ireact = myorder(kreact)
c
c     f0
c
      write(16,10) what(ireact)
   10 format(a8,' f0=1')
      do igroup=1,ngroup
      write(16,20) eav(igroup),xcmom(0,ireact,igroup)
   20 format(1p2d11.4)
      enddo
      write(16,30)
   30 format(30x,'(blank line)')
c
c     f1
c
      write(16,40) what(ireact)
   40 format(a8,' f1')
      do igroup=1,ngroup
      write(16,20) eav(igroup),xcmom(1,ireact,igroup)
      enddo
      write(16,30)
c-----fit
      write(16,50) what(ireact)
   50 format(a8,' f1-fit')
      do igroup=1,ngroup
      if(eav(igroup).ge.e1.and.
     1   eav(igroup).lt.e2) then
      write(16,20) eav(igroup),xcnew(1,ireact,igroup)
      endif
      enddo
      write(16,30)
c
c     f2
c
      write(16,60) what(ireact)
   60 format(a8,' f2')
      do igroup=1,ngroup
      write(16,20) eav(igroup),xcmom(2,ireact,igroup)
      enddo
      write(16,30)
c-----fit
      write(16,70) what(ireact)
   70 format(a8,' f2-fit')
      do igroup=1,ngroup
      if(eav(igroup).ge.e1.and.
     1   eav(igroup).lt.e2) then
      write(16,20) eav(igroup),xcnew(2,ireact,igroup)
      endif
      enddo
      write(16,30)
c
c     Optionally Copy 2 NJOY results = f1 and f2
c
      if(njoycur.gt.0) then
      do 110 loop=1,2
   80 read(14,90) line60
   90 format(a60)
      if(line22.eq.'      ') go to 100
      write(16,90) line60
      go to 80
  100 write(16,30)
  110 continue
      endif
c
c     reactions.
c
      enddo
      return
      end
      subroutine mfmt2152
c=======================================================================
c
c     Output MF/MT=2/152
c
c=======================================================================
      include 'implicit.h'
      include 'urrfit.h'
      character*80 label
      common/header/c1h,c2h,l1h,l2h,n1h,n2h,math,mfh,mth,noseq
      common/leader/c1, c2, l1, l2, n1, n2
      common/njoycom/njoycur,nunr,LSSF,INTUNR,ICOMP
      common/urrlimits/elow,ehigh
c-----Define MF and MT
      mfh = 2
      mth = 152
c-----Initialize HEADER and LEADER
      l1h = LSSF     ! LSSF = Read as Input
      l2h = 0
      n1h = 0
      n2h = INTUNR   ! INTUNR = interpolation code = NJOY uses 2 or 5?
      l1  = 5        ! always 5 reactions (T, E, F, C, Heat)
      l2  = 2        ! always 2 sigma-0 values
c     n1  = 0        ! # of words = defined below
      n2  = nunr     ! variable number of energies
c--------------------------------------------------------
      nreac   = l1
      nsigz   = l2
      sigz(1) = 1.0d10
      sigz(2) = 0.0d0
c-----set first and last energies to URR energy limits
      eunr(1)      = elow
      eunr(nunr)   = ehigh
c-----------------------------------------------------------------------
c
c     Format.
c
c-----------------------------------------------------------------------
      ii = 0
      do isigz = 1,nsigz
      ii = ii + 1
c     sigz(isigz) = x(ii)
      x(ii) = sigz(isigz)
      enddo
c-----shield x/c = vs. energy range, reaction, sigma-0
      do iunr = 1,nunr                ! energy range
      ii = ii + 1
c     eunr(iunr) = x(ii)
      x(ii) = eunr(iunr)
      do ireac = 1,nreac              ! reaction
      do isigz = 1,nsigz              ! sigma0
      ii = ii + 1
c     sigu(ireac,isigz,iunr) = x(ii)
      x(ii) = sigu(ireac,isigz,iunr)
      enddo
      enddo
      enddo
c-----------------------------------------------------------------------
c
c     Output in ENDF format
c
c-----------------------------------------------------------------------
      n1 = ii
      write(*,10) n1,nreac,nsigz,nunr,LSSF,INTUNR
   10 format(' As Written'/
     1       ' n1     =',i5/
     2       ' nreac  =',i5/
     3       ' nsigz  =',i5/
     4       ' nunr   =',i5/
     4       ' LSSF   =',i5/
     4       ' INTUNR =',i5)
      call conto
      call cardo(c1,c2,l1,l2,n1,n2)
      call listo9(x(1),n1)
      call outs(math,mfh)
      return
      end
      subroutine mfmt2153
c=======================================================================
c
c     Output MF/MT=2/153
c
c=======================================================================
      include 'implicit.h'
      include 'urrfit.h'
      character*80 label
      common/header/c1h,c2h,l1h,l2h,n1h,n2h,math,mfh,mth,noseq
      common/leader/c1, c2, l1, l2, n1, n2
      common/njoycom/njoycur,nunr,LSSF,INTUNR,ICOMP
      common/urrlimits/elow,ehigh
c-----Define MF and MT
      mfh = 2
      mth = 153
c-----WARNING - based on LSSF
      if(LSSF.eq.0) then
      write(3,10)
      write(*,10)
   10 format(1x,71('-')/
     1 ' WARNING - LSSF = 0 - Output is Cross Sections, not f-factors.'/
     2 1x,71('-'))
      else
      write(3,20)
      write(*,20)
   20 format(1x,71('-')/
     1 ' WARNING - LSSF = 1 - Output is f-factors, not Cross Sections.'/
     2 1x,71('-'))
      endif
c-----Initialize HEADER and LEADER
      l1h = 0
      l2h = 0
      n1h = INTUNR   ! not defined in NJOY manual = guess INTERP code
      n2h = 2        ! Always 2 bins (bands)
      l1  = LSSF     ! LSSF  = Read as Input
      l2  = ICOMP    ! ICOMP = what NJOY uses
c     n1  = 0        ! # of words - defined below
      n2  = nunr     ! # of energies
c-----------------------------------------------------------------------
c
c     Always output 2 band GROUPIE results.
c
c-----------------------------------------------------------------------
c-----set first and last energies to URR energy limits
      eunr(1)      = elow
      eunr(nunr)   = ehigh
c-----------------------------------------------------------------------
c
c     Output multi-band Data.
c
c-----------------------------------------------------------------------
      nbin = n2h                ! always 2 bins
      ii = 0
      do iunr=1,nunr
      ii = ii + 1
c     eunr(iunr) = x(ii)        ! lower energy
      x(ii) = eunr(iunr)        ! lower energy
      do ipar=1,6               ! 6 parameters=P,T,E,F,C,H
      do ibin=1,nbin            ! nbin bins
      ii = ii + 1
c     param(ipar,ibin,iunr) = x(ii)
      x(ii) = bands(ibin,ipar,iunr)
      enddo
      enddo
      enddo
c-----------------------------------------------------------------------
c
c     Output in ENDF format
c
c-----------------------------------------------------------------------
      n1 = ii
      write(*,30) n1,nbin,nunr,LSSF,INTUNR,ICOMP
   30 format(' As Written'/
     1       ' n1     =',i5/
     2       ' nbin   =',i5/
     3       ' nunr   =',i5/
     3       ' LSSF   =',i5/
     3       ' INTUNR =',i5/
     3       ' ICOMP  =',i5)
      call conto
      call cardo(c1,c2,l1,l2,n1,n2)
      call listo9(x(1),n1)
      call outs(math,mfh)
      return
      end
      SUBROUTINE ENDIT
C=======================================================================
C
C     NORMAL END - Print code name but NO TIME.
C
C=======================================================================
      IMPLICIT REAL*4 (A-H,O-Z)  ! note - REAL*4 not 8
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      CHARACTER*8 CODENAME
      COMMON/NAMECODE/CODENAME
      WRITE(3,10) CODENAME
      WRITE(*,10) CODENAME
      STOP
C=======================================================================
C
C     ERROR END - Print ERROR message but NO TIME.
C
C=======================================================================
      ENTRY ENDERROR
      WRITE(3,20)
      WRITE(*,20)
   10 FORMAT(1X,71('=')/1X,A8/1X,71('='))
   20 FORMAT(1X,71('=')/1X,'   ERROR'/1X,71('='))
      STOP
      END
      SUBROUTINE CONTO
C=======================================================================
C
C     WRITE ONE ENDF/B CONTROL RECORD.
C
C=======================================================================
      INCLUDE 'implicit.h'
      INTEGER*4 OUTP,OTAPE
      CHARACTER*1 FIELD2
      COMMON/ENDFIO/INP,OUTP,ITAPE,OTAPE
      COMMON/HEADER/C1H,C2H,L1H,L2H,N1H,N2H,MATH,MFH,MTH,NOSEQ
      DIMENSION FIELD2(11,2)
C-----LAST MAT NUMBER - USED TO RESET SEQUENCE NUMBER
      DATA LASTMAT/-100000/
C-----NO OUTPUT IF OUTPUT UNIT IS TURNED OFF
      IF(OTAPE.LE.0) RETURN
C-----IF SEND, FEND OR MEND OUTPUT IN STANDARD FORM.
      IF(MTH.GT.0) GO TO 10
      CALL OUTS(MATH,MFH)
      RETURN
C-----CONVERT FLOATING POINT NUMBERS TO STANDARD OUTPUT FORM.
   10 CALL OUT9(C1H,FIELD2(1,1))
      CALL OUT9(C2H,FIELD2(1,2))
C-----ELIMINATE -0
      IF(IABS(L1H).LE.0) L1H=0
      IF(IABS(L2H).LE.0) L2H=0
      IF(IABS(N1H).LE.0) N1H=0
      IF(IABS(N2H).LE.0) N2H=0
C-----IF NEW MAT RESET SEQUENCE NUMBER
      IF(MATH.eq.LASTMAT) go to 20
      LASTMAT=MATH
      NOSEQ=1
C-----OUTPUT LINE IMAGE.
   20 IF(NOSEQ.LE.0) NOSEQ=1
      WRITE(OTAPE,30) FIELD2,L1H,L2H,N1H,N2H,MATH,MFH,MTH,NOSEQ
      NOSEQ=NXTSEQ(NOSEQ)
      RETURN
   30 FORMAT(22A1,4I11,I4,I2,I3,I5)
      END
      SUBROUTINE CARDO(C1,C2,L1,L2,N1,N2)
C=======================================================================
C
C     WRITE ENDF/B LINE.
C
C=======================================================================
      INCLUDE 'implicit.h'
      INTEGER*4 OUTP,OTAPE
      CHARACTER*1 FIELD2
      COMMON/ENDFIO/INP,OUTP,ITAPE,OTAPE
      COMMON/HEADER/C1H,C2H,L1H,L2H,N1H,N2H,MATH,MFH,MTH,NOSEQ
      DIMENSION FIELD2(11,2)
C-----NO OUTPUT IF OUTPUT UNIT IS TURNED OFF
      IF(OTAPE.LE.0) RETURN
      IF(MTH.GT.0) GO TO 10
      CALL OUTS(MATH,MFH)
      RETURN
C-----CONVERT FLOATING POINT NUMBERS TO STANDARD OUTPUT FORM.
   10 CALL OUT9(C1,FIELD2(1,1))
      CALL OUT9(C2,FIELD2(1,2))
C-----ELIMINATE -0
      IF(IABS(L1).LE.0) L1=0
      IF(IABS(L2).LE.0) L2=0
      IF(IABS(N1).LE.0) N1=0
      IF(IABS(N2).LE.0) N2=0
C-----OUTPUT LINE.
      IF(NOSEQ.LE.0) NOSEQ=1
      WRITE(OTAPE,20) FIELD2,L1,L2,N1,N2,MATH,MFH,MTH,NOSEQ
      NOSEQ=NXTSEQ(NOSEQ)
      RETURN
   20 FORMAT(22A1,4I11,I4,I2,I3,I5)
      END
      SUBROUTINE LISTI(X,IX)
C=======================================================================
C
C     READ LIST DATA.
C
C=======================================================================
      INCLUDE 'implicit.h'
      INTEGER*4 OUTP,OTAPE
      CHARACTER*1 FIELD6
      COMMON/ENDFIO/INP,OUTP,ITAPE,OTAPE
      DIMENSION X(IX),FIELD6(11,6)
C
C     READ LIST RECORD PARAMETERS
C
C-----SET UP LOOP OVER CARDS.
      DO 20 I1=1,IX,6
      I2=I1+5
      IF(I2.GT.IX) I2=IX
C-----READ AS CHARACTERS
      READ(ITAPE,30) FIELD6
C-----CONVERT FROM CHARACTERS TO FLOATING POINT
      K=0
      DO 10 L=I1,I2
      K=K+1
   10 CALL IN9(X(L),FIELD6(1,K))
   20 CONTINUE
      RETURN
   30 FORMAT(66A1)
      END
      SUBROUTINE LISTO9(X,IX)
C=======================================================================
C
C     WRITE LIST DATA
C
C     SAME AS LISTO, BUT USE OUT9G NOT OUT10.
C
C=======================================================================
      INCLUDE 'implicit.h'
      INTEGER*4 OUTP,OTAPE
      CHARACTER*1 FIELD6
      COMMON/ENDFIO/INP,OUTP,ITAPE,OTAPE
      COMMON/HEADER/C1H,C2H,L1H,L2H,N1H,N2H,MATH,MFH,MTH,NOSEQ
      DIMENSION X(IX),FIELD6(11,6)
C-----NO OUTPUT IF OUTPUT UNIT IS TURNED OFF
      IF(OTAPE.LE.0) RETURN
C-----NOTHING TO DO IF NO POINTS TO OUTPUT
      IF(IX.LE.0) RETURN
C-----SET UP LOOP OVER CARDS.
      DO 80 I1=1,IX,6
      I2=I1+5
      IF(I2.GT.IX) I2=IX
C-----CONVERT DATA TO NORMAL FORM.
      K=0
      DO 10 L=I1,I2
      K=K+1
c-----USE OUT9G - NOT OUT10.
   10 CALL OUT9G(X(L),FIELD6(1,K))
C-----OUTPUT ONE LINE.
      GO TO (20,30,40,50,60,70),K
   20 WRITE(OTAPE,90)   (FIELD6(M,1),M=1,11),
     1 MATH,MFH,MTH,NOSEQ
      GO TO 80
   30 WRITE(OTAPE,100) ((FIELD6(M,II),M=1,11),II=1,2),
     1 MATH,MFH,MTH,NOSEQ
      GO TO 80
   40 WRITE(OTAPE,110) ((FIELD6(M,II),M=1,11),II=1,3),
     1 MATH,MFH,MTH,NOSEQ
      GO TO 80
   50 WRITE(OTAPE,120) ((FIELD6(M,II),M=1,11),II=1,4),
     1 MATH,MFH,MTH,NOSEQ
      GO TO 80
   60 WRITE(OTAPE,130) ((FIELD6(M,II),M=1,11),II=1,5),
     1 MATH,MFH,MTH,NOSEQ
      GO TO 80
   70 WRITE(OTAPE,140) ((FIELD6(M,II),M=1,11),II=1,6),
     1 MATH,MFH,MTH,NOSEQ
   80 NOSEQ=NXTSEQ(NOSEQ)
      RETURN
   90 FORMAT(11A1,55X,I4,I2,I3,I5)
  100 FORMAT(22A1,44X,I4,I2,I3,I5)
  110 FORMAT(33A1,33X,I4,I2,I3,I5)
  120 FORMAT(44A1,22X,I4,I2,I3,I5)
  130 FORMAT(55A1,11X,I4,I2,I3,I5)
  140 FORMAT(66A1    ,I4,I2,I3,I5)
      END
      SUBROUTINE OUTT
C=======================================================================
C
C     OUTPUT TEND, MEND, FEND OR SEND RECORDS.
C     ENTRY POINTS ARE,
C     OUTT = OUTPUT TEND RECORD
C     OUTM = OUTPUT MEND RECORD
C     OUTF = OUTPUT FEND RECORD
C     OUTS = OUTPUT SEND RECORD
C
C     OUTPUT TEND RECORD
C
C=======================================================================
      INCLUDE 'implicit.h'
      INTEGER*4 OUTP,OTAPE
      COMMON/ENDFIO/INP,OUTP,ITAPE,OTAPE
      COMMON/HEADER/C1H,C2H,L1H,L2H,N1H,N2H,MATH,MFH,MTH,NOSEQ
C-----NO OUTPUT IF OUTPUT UNIT IS TURNED OFF
      IF(OTAPE.LE.0) RETURN
      WRITE(OTAPE,10)
      RETURN
C=======================================================================
C
C     OUTPUT MEND RECORD
C
C=======================================================================
      ENTRY OUTM
C-----NO OUTPUT IF OUTPUT UNIT IS TURNED OFF
      IF(OTAPE.LE.0) RETURN
      WRITE(OTAPE,20)
C-----RESET NOSEQ AFTER MEND OUTPUT
      NOSEQ=1
      RETURN
C=======================================================================
C
C     OUTPUT FEND RECORD
C
C=======================================================================
      ENTRY OUTF(MATOUT)
C-----NO OUTPUT IF OUTPUT UNIT IS TURNED OFF
      IF(OTAPE.LE.0) RETURN
      WRITE(OTAPE,30) MATOUT
      NOSEQ=1
      RETURN
C=======================================================================
C
C     OUTPUT SEND RECORD
C
C=======================================================================
      ENTRY OUTS(MATOUT,MFOUT)
C-----NO OUTPUT IF OUTPUT UNIT IS TURNED OFF
      IF(OTAPE.LE.0) RETURN
C-----NOSEQ = 0 ON FEND/MEND/TEND LINE
      IF(MFOUT.LE.0) then
      WRITE(OTAPE,30) MATOUT
      else
      WRITE(OTAPE,40) MATOUT,MFOUT  ! NOSEQ = 99999
      endif
      NOSEQ=1
      RETURN
   10 FORMAT(66X,'  -1 0  0    0')
   20 FORMAT(66X,'   0 0  0    0')
   30 FORMAT(66X,I4, ' 0  0    0')
   40 FORMAT(66X,I4,I2,'  099999')
      END
      SUBROUTINE IN9(E,FIELD)
C=======================================================================
C
C     PURPOSE
C     =======
C     CONVERT FROM HOLLERITH TO FLOATING POINT.
C     12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT (I Thank Viktor Zerkin)
C                  More than 3 digit exponent = ERROR - return 0.0
C
C     ARGUMENTS
C     =========
C     E       = FLOATING POINT NUMBER (OUTPUT)
C     FIELD   = 11A1 CHARACTER STRING (INPUT)
C
C     METHOD
C     ======
C     FIELD IS A STRING OF 11 CHARACTERS.
C     IT IS CONVERTED INTO A FLOATING POINR NUMBER (E)
C
C=======================================================================
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      save
      CHARACTER*1 MESS,DIGIT,FIELD,IFIELD
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENTS
      DIMENSION FIELD(11),TENS(-99:99),DIGIT(0:9),MESS(11),XDIG(0:9)
      DATA MESS/' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '/
      DATA DIGIT/'0','1','2','3','4','5','6','7','8','9'/
      DATA XDIG/0.0D+00,1.0D+00,2.0D+00,3.0D+00,4.0D+00,
     1          5.0D+00,6.0D+00,7.0D+00,8.0D+00,9.0D+00/
C-----ON FIRST CALL DEFINE POWERS OF 10
      DATA IPASS/0/
      IF(IPASS.NE.0) GO TO 20
      IPASS=1
      TENS(0)=1.0D+00
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENTS
      DO 10 I=1,99
      TENS( I)=TENS(I-1)*10.0D+00
   10 TENS(-I)=TENS(1-I)/10.0D+00
C
C     TRANSLATE MANTISSA.
C
C-----SKIP LEADING BLANK CHARACTERS.
   20 DO 30 I=1,11
      IF(FIELD(I).NE.' ') GO TO 40
   30 CONTINUE
C-----FIELD IS COMPLETELY BLANK. RETURN ZERO.
      E=0.0D+00
      RETURN
C-----INITIALIZE SIGN TO PLUS AND THEN CHECK FOR LEADING MINUS SIGN.
   40 SIGN=1.0D+00
C-----06/02/09 - ADDED TEST FOR LEADING + SIGN.
      IF(FIELD(I).NE.'-') GO TO 50
      SIGN=-1.0D+00
      I=I+1
c-----added leading +
      GO TO 60
   50 IF(FIELD(I).EQ.'+') I = I + 1
C-----INITIALIZE FIXED POINT INPUT FIELD AND POSITION OF DECIMAL POINT.
   60 X9IN=0.0
      IPT=-20
      IMZERO=0
C-----SCAN REMAINDER OF MANTISSA.
      DO 120 J=I,11
      IFIELD=FIELD(J)
C-----SCAN FOR DIGIT OR DECIMAL POINT (WHICH ARE PART OF MANTISSA).
      DO 70 K=0,9
      IF(IFIELD.EQ.DIGIT(K)) GO TO 90
   70 CONTINUE
      IF(IFIELD.NE.'.') GO TO 80
      IPT=0
      GO TO 120
C-----SCAN FOR BLANK (WHICH ENDS MANTISSA).
   80 IF(IFIELD.EQ.' ') GO TO 130
C-----SCAN FOR e, E, d, D, - OR + (WHICH BEGINS EXPONENT).
      IF(IFIELD.EQ.'e'.OR.IFIELD.EQ.'E') GO TO 160
      IF(IFIELD.EQ.'d'.OR.IFIELD.EQ.'D') GO TO 160
      IF(IFIELD.EQ.'-') GO TO 190
      IF(IFIELD.EQ.'+') GO TO 170
C-----ERROR. CANNOT IDENTIFY CHARACTER.
      GO TO 240
C-----DIGIT FOUND. SAVE TRAILING ZEROES AFTER DECIMAL POINT.
   90 IF(IPT.LT.0) GO TO 110
      IF(K.NE.0) GO TO 100
C-----SAVE TRAILING ZEROES.
      IMZERO=IMZERO+1
      GO TO 120
  100 IF(IMZERO.LE.0) GO TO 110
C-----INSERT ZEROES BEFORE NEXT NUMBER.
      X9IN=10.0D+00*X9IN
      IPT=IPT+1
      IMZERO=IMZERO-1
      GO TO 100
C-----DIGIT FOUND. INCREMENT FIXED POINT EQUIVALENT AND DECIMAL POINT
C-----OFFSET.
  110 X9IN=10.0D+00*X9IN+XDIG(K)
      IPT=IPT+1
  120 CONTINUE
C-----ENTIRE FIELD TRANSLATED (NO EXPONENT). CONVERT TO FLOATING POINT.
      GO TO 150
C-----BLANK FOUND (END OF MANTISSA). SCAN REMAINDER OF FIELD FOR
C-----EXPONENT.
  130 I=J+1
      IF(I.GT.11) GO TO 150
      DO 140 J=I,11
      IFIELD=FIELD(J)
      IF(IFIELD.EQ.' ') GO TO 140
      IF(IFIELD.EQ.'e'.OR.IFIELD.EQ.'E') GO TO 160
      IF(IFIELD.EQ.'d'.OR.IFIELD.EQ.'D') GO TO 160
      IF(IFIELD.EQ.'-') GO TO 190
      IF(IFIELD.EQ.'+') GO TO 170
C-----ERROR. CANNOT IDENTIFY CHARACTER.
      GO TO 240
  140 CONTINUE
C-----ENTIRE FIELD TRANSLATED (NO EXPONENT). CONVERT TO FLOATING POINT.
  150 E=X9IN
      IF(IPT.GT.0) E=E/TENS(IPT)
      E=SIGN*E
      RETURN
C
C     TRANSLATE EXPONENT.
C
C-----BEGINNING OF EXPONENT FOUND (E OR D). CHECK FOR FOLLOWING - OR +.
  160 J=J+1
      IFIELD=FIELD(J)
      IF(IFIELD.EQ.'-') GO TO 190
      IF(IFIELD.NE.'+') GO TO 180
C----- + FOUND. INITIALIZE EXPONENT SIGN.
  170 J=J+1
  180 ISIGN=1
      GO TO 200
C----- - FOUND. INITIALIZE EXPONENT SIGN.
  190 J=J+1
      ISIGN=-1
C-----INITIALIZE EXPONENT AND SCAN REMAINING CHARACTERS FOR EXPONENT.
  200 IEXP=0
      DO 230 I=J,11
      IFIELD=FIELD(I)
      IF(IFIELD.EQ.' ') GO TO 230
      DO 210 K=0,9
      IF(IFIELD.EQ.DIGIT(K)) GO TO 220
  210 CONTINUE
C-----ERROR. CANNOT IDENTIFY CHARACTER.
      GO TO 240
C-----DIGIT FOUND. INCREMENT EXPONENT.
C-----OFFSET.
  220 IEXP=10*IEXP+K
  230 CONTINUE
C-----ENTIRE FIELD TRANSLATED (WITH EXPONENT). CONVERT TO FLOATING
C-----POINT.
      E=X9IN
      IEXP=ISIGN*IEXP
      IF(IPT.GT.0) IEXP=IEXP-IPT
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENTS
C-----11/22/2013 - Restricted to 2 Digit Eexponent
      IF(iabs(IEXP).GT.99) GO TO 240
      E=SIGN*E*TENS(IEXP)
      RETURN
C
C     ERROR CONDITIONS.
C
C-----ILLEGAL CHARACTER.
  240 MESS(J)='*'
      write(*,250) FIELD,MESS
  250 FORMAT(1X,11A1/1X,11A1/' ERROR in Input Data...Translated as 0.')
      E=0.0D+00
      MESS(J)=' '
      RETURN
      END
      SUBROUTINE OUT9(ZIN,FIELD)
C=======================================================================
C
C     PURPOSE
C     =======
C     FORMAT NUMBER FOR OUTPUT TO INCLUDE AS MANY DIGITS OF
C     ACCURACY AS POSSIBLE.
C     12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT
C                  No test for 4 or more digit exponent
C     10/25/2014 - Changed from D to E exponential form to improve
C                  compatibility between computer languages
C
C     040923 - CHANGED ZLOW FROM 1.0D-4 TO 1.0D-3
C              NEAR 1.0D-3 PRECISION GOES FROM 999,999 TO 1,000,000
C              FOR SMOOTHEST TRANSITION FROM 6 TO 7 DIGITS
C
C     ARGUMENTS
C     =========
C     Z        = FLOATING POINT NUMBER OF BE OUTPUT (INPUT)
C     FIELD    = 11A1 CHARACTERS TO OUTPUT          (OUTPUT)
C
C     METHOD
C     ======
C     COLUMNS            12345678901     ACCURACY
C     -------------------------------------------
C     0 TO 10^-9          1.2345E-12     5 DIGITS
C     10^-9 TO 10^-3      1.23456E-8     6 DIGITS
C     10^-3 TO 10^-2      .001234567     7 DIGITS
C     10^-2 TO 10^-1      .012345678     8 DIGITS
C     10^-1 TO 1          .123456789     9 DIGITS
C     1 TO 10^9           12345.6789     9 DIGITS
C     10^9 TO 10^10       1.23456E+9     6 DIGITS
C     10^10 >             1.2345E+12     5 DIGITS
C
C     OUTPUT WILL BE IN 11 COLUMN FORMAT
C
C     WARNING - THIS IS NOT A GENERAL ROUNDING ROUTINE WHICH WILL WORK
C               FOR ROUNDING TO ANY NUMBER OF DIGITS - IT WILL ONLY
C               WORK PROPERLY FOR THE RANGES INDICATED ABOVE, FOR
C               11 COLUMN OUTPUT.
C
C=======================================================================
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      save
c-----INTEGER*8 for long integers
      integer*8 INMANT,INMAN9,INMAN8,INMAN7,INMAN6,INMAN5,INMAN4
      CHARACTER*1 FIELD,DIGITS,ZEROH
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT
      DIMENSION DIGITS(0:9),FIELD(11),ZEROH(11),
     1 TENS(-99:99),ROUNDER(-99:99)
      DATA DIGITS/
     1 '0','1','2','3','4','5','6','7','8','9'/
C-----RETURN FOR = 0
      DATA ZEROH/
     1 ' ','0','.','0',' ',' ',' ',' ',' ',' ',' '/
C-----LOWER TRANSITION POINT FROM 7 TO 9 DIGIT OUTPUT
      DATA ZLOW/1.0D-03/
C-----UPPER TRANSITION POINT FROM 9 TO 7 DIGIT OUTPUT
      DATA ZHIGH/1.0D+09/
      DATA TENTH/1.0D-01/
c-----------------------------------------------------------------------
c
c     ON FIRST CALL INITIALIZE POWERS OF 10
c
c-----------------------------------------------------------------------
      DATA IPASS/0/
      IF(IPASS.NE.0) GO TO 50
      IPASS=1
c-----INTEGER*8 for long integers
      INMAN4  = 10000
      INMAN5  = 10*INMAN4
      INMAN6  = 10*INMAN5
      INMAN7  = 10*INMAN6
      INMAN8  = 10*INMAN7
      INMAN9  = 10*INMAN8
      TENS(0)=1.0D+00
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT
C-----11/22/2013 - Restricted to 2 Digit Exponent
      DO 10 I=1,99
      TENS( I)=TENS(I-1)*10.0D+00
      TENS(-I)=TENS(1-I)/10.0D+00
   10 ROUNDER(I)  = 5.001D-05
      DO 20 I=-99,-10
   20 ROUNDER(I)  = 5.001D-05
      DO 30 I=-9,-4
   30 ROUNDER(I)  = 5.001D-06
      ROUNDER(-3) = 5.001D-07
      ROUNDER(-2) = 5.001D-08
      ROUNDER(-1) = 5.001D-09
      ROUNDER( 0) = 5.001D-09
      DO 40 I=1,8
   40 ROUNDER(I)  = 5.001D-09
      ROUNDER(9)  = 5.001D-06
c-----------------------------------------------------------------------
C
C     IMMEDIATELY RETURN 0.00000+00.
C
c-----------------------------------------------------------------------
C-----02/14/04 - ADDED, JUST IN CASE
   50 IF(DABS(ZIN).le.0.0D+0) GO TO 60
      IF(DABS(ZIN).lt.1.0D-99) GO TO 60    ! Only 2 digit exponents
      IF(DABS(ZIN).gt.1.0D+99) GO TO 60
c----- ZIN = 0 handled above
      IF(ZIN.lt.0.0d+0) go to 80
      go to 90
c-----Return 0
   60 DO 70 I=1,11
   70 FIELD(I)=ZEROH(I)
      RETURN
c-----------------------------------------------------------------------
C
C     DEFINE SIGN OF MANTISSA AND ABSOLUTE MANTISSA
C
c-----------------------------------------------------------------------
C-----NEGATIVE.
   80 FIELD(1)='-'
      Z=-ZIN
      GO TO 100
C-----POSITIVE.
   90 FIELD(1)=' '
      Z=ZIN
c-----------------------------------------------------------------------
c
c     DEFINE EXPONENT AND NORMALIZED MANTISSA
c
c-----------------------------------------------------------------------
  100 IEXP=DLOG10(Z)
      IF(Z.LT.1.0D+00) IEXP = IEXP - 1
c-----11/22/2013 - Decide here on F or E Output
c-----11/26/2013 - Restrict to 2 digit exponent - may change by 1 or 2
      if(IABS(IEXP).gt.97) go to 60
      if(ZIN.lt.0.0D+0) then
      ZN=Z*TENS(-IEXP) + ROUNDER(IEXP)       ! Standard
      else
      if(Z.LE.ZLOW.OR.Z.GE.ZHIGH) then       ! F or E Format?
      ZN=Z*TENS(-IEXP) + TENTH*ROUNDER(IEXP) ! Extra Digit
      else
      ZN=Z*TENS(-IEXP) + ROUNDER(IEXP)       ! Standard
      endif
      endif
      IF(ZN.eq.1.0D+00) go to 120            ! Test rounding underflow
      IF(ZN.gt.1.0D+00) go to 110            ! Test rounding underflow
      IEXP=IEXP-1         ! MUST be < 1
      GO TO 120
  110 IF(ZN.lt.10.0D+00) go to 120           ! Test rounding overflow
      IEXP=IEXP+1         ! MUST be >= 10
      ZN=ZN/10.0D+00
  120 Z = ZN*TENS(IEXP)
c-----------------------------------------------------------------------
C
C     SELECT F OR E FORMAT
C
c-----------------------------------------------------------------------
      IF(Z.LE.ZLOW.OR.Z.GE.ZHIGH) GO TO 150
c-----------------------------------------------------------------------
C
C     F FORMAT
C
C     12345678901
C      X.XXXXXXXX = 9 DIGITS
C      .001234567
C      123456789.
C
c-----------------------------------------------------------------------
C-----DEFINE 6 TO 9 DIGIT MANTISSA WITH ROUNDING
      IPOWER=8-IEXP
      IF(IEXP.LT.0) IPOWER=8
      INMANT=Z*TENS(IPOWER)
C-----CHECK FOR OVERFLOW DUE TO ROUNDING
      if(INMANT.lt.INMAN9) go to 130
      INMANT=INMAN8
      IEXP=IEXP+1
C-----DECIMAL POINT.
  130 IDOT=3+IEXP
      IF(IDOT.LE.2) THEN
C----- IF < 1, MOVE DECIMAL POINT TO COLUMN 2 AND ADD A DIGIT
      IDOT=2
      INMANT=Z*10.0D+00*TENS(IPOWER)
      ENDIF
      FIELD(IDOT)='.'
C-----MANTISSA - LAST DIGIT TO FIRST.
      II=11
      DO 140 I=2,11
      IF(II.EQ.IDOT) GO TO 140
      I2=INMANT/10
      I3=INMANT-10*I2
      FIELD(II)=DIGITS(I3)
      INMANT=I2
  140 II=II-1
      RETURN
c-----------------------------------------------------------------------
C
C     E FORMAT
C
C     12345678901
C      X.XXXXE+NN = 5 DIGITS
C      X.XXXXXE+N = 6 DIGITS
C
C     11/22/2013 - If not negative, use first column
C     12345678901
C     X.XXXXXE+NN = 6 DIGITS
C     X.XXXXXXE+N = 7 DIGITS
c
c-----------------------------------------------------------------------
C-----Negative?
  150 IF(ZIN.lt.0.0d+0) go to 170
c
c     POsitive. Use first column - Decimal point is always in column 2
c
      FIELD(2)='.'
      KDOT    = 2
      ISTART  = 1
      IF(IABS(IEXP).GE.10) GO TO 160
      ID=8                               ! 1 Digit exponent
      INMANT=(1.0D+06)*ZN
      IF(INMANT.lt.INMAN7) go to 190
      INMANT=INMAN6
      IEXP=IEXP+1
      IF(IABS(IEXP).LT.10) GO TO 190
  160 ID=7                               ! 2 Digit exponent
      INMANT=(1.0D+05)*ZN
C-----CHECK FOR OVERFLOW DUE TO ROUNDING
      IF(INMANT.lt.INMAN6) go to 190
      INMANT=INMAN5
      IEXP=IEXP+1
c
c     Negative Number - Cannot use first column
c
C-----DECIMAL POINT IS ALWAYS IN COLUMN 3
  170 FIELD(3)='.'
      KDOT    = 3
      ISTART  = 2
      IF(IABS(IEXP).GE.10) GO TO 180
      ID=8                                 ! 1 Digit Exponent
      INMANT=(1.0D+05)*ZN
      IF(INMANT.lt.INMAN6) go to 190
      INMANT=INMAN5
      IEXP=IEXP+1
      IF(IABS(IEXP).LT.10) GO TO 190
  180 ID=7                                 ! 2 Digit Exponent
      INMANT=(1.0D+04)*ZN
      IF(INMANT.lt.INMAN5) go to 190
      INMANT=INMAN4
      IEXP=IEXP+1
C
C     DEFINE MANTISSA
C
  190 IEXPS=ID+1
      II=ID
      DO 200 I=ISTART,ID
      IF(II.EQ.KDOT) GO TO 200
      I2=INMANT/10
      I3=INMANT-10*I2
      FIELD(II)=DIGITS(I3)
      INMANT=I2
  200 II=II-1
C
C     E
C
      FIELD(IEXPS) = 'E'
      IEXPS = IEXPS + 1
C
C     SIGN OF EXPONENT
C
      IF(IEXP.ge.0) go to 210
      IEXP=-IEXP
      FIELD(IEXPS)='-'
      GO TO 220
  210 FIELD(IEXPS)='+'
C
C     EXPONENT
C
  220 IF(IEXP.lt.10) go to 230
      KEXP=IEXP/10
      FIELD(10)=DIGITS(KEXP)
      IEXP=MOD(IEXP,10)
  230 FIELD(11)=DIGITS(IEXP)
c
c     If using column 1 but mantissa ends in 0, move mantissa right.
c
      if(KDOT.eq.2.and.FIELD(ID).eq.'0') then
      do k=ID-1,1,-1
      FIELD(k+1)=FIELD(k)
      enddo
      FIELD(1) = ' '
      endif
      RETURN
      END
      SUBROUTINE OUT9G(ZIN,FIELD)
C=======================================================================
C
C     PURPOSE
C     =======
C     20915/7/28 - same as OUT9, but column 1 is always '-' or blank.
C                  Used by GROUPIE to simplify listings.
C
C     FORMAT NUMBER FOR OUTPUT TO INCLUDE AS MANY DIGITS OF
C     ACCURACY AS POSSIBLE.
C     12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT
C                  No test for 4 or more digit exponent
C     10/25/2014 - Changed from D to E exponential form to improve
C                  compatibility between computer languages
C
C     040923 - CHANGED ZLOW FROM 1.0D-4 TO 1.0D-3
C              NEAR 1.0D-3 PRECISION GOES FROM 999,999 TO 1,000,000
C              FOR SMOOTHEST TRANSITION FROM 6 TO 7 DIGITS
C
C     ARGUMENTS
C     =========
C     Z        = FLOATING POINT NUMBER OF BE OUTPUT (INPUT)
C     FIELD    = 11A1 CHARACTERS TO OUTPUT          (OUTPUT)
C
C     METHOD
C     ======
C     COLUMNS            12345678901     ACCURACY
C     -------------------------------------------
C     0 TO 10^-9          1.2345E-12     5 DIGITS
C     10^-9 TO 10^-3      1.23456E-8     6 DIGITS
C     10^-3 TO 10^-2      .001234567     7 DIGITS
C     10^-2 TO 10^-1      .012345678     8 DIGITS
C     10^-1 TO 1          .123456789     9 DIGITS
C     1 TO 10^9           12345.6789     9 DIGITS
C     10^9 TO 10^10       1.23456E+9     6 DIGITS
C     10^10 >             1.2345E+12     5 DIGITS
C
C     OUTPUT WILL BE IN 11 COLUMN FORMAT
C
C     WARNING - THIS IS NOT A GENERAL ROUNDING ROUTINE WHICH WILL WORK
C               FOR ROUNDING TO ANY NUMBER OF DIGITS - IT WILL ONLY
C               WORK PROPERLY FOR THE RANGES INDICATED ABOVE, FOR
C               11 COLUMN OUTPUT.
C
C=======================================================================
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      save
c-----INTEGER*8 for long integers
      integer*8 INMANT,INMAN9,INMAN8,INMAN7,INMAN6,INMAN5,INMAN4
      CHARACTER*1 FIELD,DIGITS,ZEROH
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT
      DIMENSION DIGITS(0:9),FIELD(11),ZEROH(11),
     1 TENS(-99:99),ROUNDER(-99:99)
      DATA DIGITS/
     1 '0','1','2','3','4','5','6','7','8','9'/
C-----RETURN FOR = 0
      DATA ZEROH/
     1 ' ','0','.','0',' ',' ',' ',' ',' ',' ',' '/
C-----LOWER TRANSITION POINT FROM 7 TO 9 DIGIT OUTPUT
      DATA ZLOW/1.0D-03/
C-----UPPER TRANSITION POINT FROM 9 TO 7 DIGIT OUTPUT
      DATA ZHIGH/1.0D+09/
      DATA TENTH/1.0D-01/
c-----------------------------------------------------------------------
c
c     ON FIRST CALL INITIALIZE POWERS OF 10
c
c-----------------------------------------------------------------------
      DATA IPASS/0/
      IF(IPASS.NE.0) GO TO 50
      IPASS=1
c-----INTEGER*8 for long integers
      INMAN4  = 10000
      INMAN5  = 10*INMAN4
      INMAN6  = 10*INMAN5
      INMAN7  = 10*INMAN6
      INMAN8  = 10*INMAN7
      INMAN9  = 10*INMAN8
      TENS(0)=1.0D+00
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT
C-----11/22/2013 - Restricted to 2 Digit Exponent
      DO 10 I=1,99
      TENS( I)=TENS(I-1)*10.0D+00
      TENS(-I)=TENS(1-I)/10.0D+00
   10 ROUNDER(I)  = 5.001D-05
      DO 20 I=-99,-10
   20 ROUNDER(I)  = 5.001D-05
      DO 30 I=-9,-4
   30 ROUNDER(I)  = 5.001D-06
      ROUNDER(-3) = 5.001D-07
      ROUNDER(-2) = 5.001D-08
      ROUNDER(-1) = 5.001D-09
      ROUNDER( 0) = 5.001D-09
      DO 40 I=1,8
   40 ROUNDER(I)  = 5.001D-09
      ROUNDER(9)  = 5.001D-06
c-----------------------------------------------------------------------
C
C     IMMEDIATELY RETURN 0.00000+00.
C
c-----------------------------------------------------------------------
C-----02/14/04 - ADDED, JUST IN CASE
   50 IF(DABS(ZIN).le.0.0D+0) GO TO 60
      IF(DABS(ZIN).lt.1.0D-99) GO TO 60    ! Only 2 digit exponents
      IF(DABS(ZIN).gt.1.0D+99) GO TO 60
c----- ZIN = 0 handled above
      IF(ZIN.lt.0.0d+0) go to 80
      go to 90
c-----Return 0
   60 DO 70 I=1,11
   70 FIELD(I)=ZEROH(I)
      RETURN
c-----------------------------------------------------------------------
C
C     DEFINE SIGN OF MANTISSA AND ABSOLUTE MANTISSA
C
c-----------------------------------------------------------------------
C-----NEGATIVE.
   80 FIELD(1)='-'
      Z=-ZIN
      GO TO 100
C-----POSITIVE.
   90 FIELD(1)=' '
      Z=ZIN
c-----------------------------------------------------------------------
c
c     DEFINE EXPONENT AND NORMALIZED MANTISSA
c
c-----------------------------------------------------------------------
  100 IEXP=DLOG10(Z)
      IF(Z.LT.1.0D+00) IEXP = IEXP - 1
c-----11/22/2013 - Decide here on F or E Output
c-----11/26/2013 - Restrict to 2 digit exponent - may change by 1 or 2
      if(IABS(IEXP).gt.97) go to 60
      ZN=Z*TENS(-IEXP) + ROUNDER(IEXP)       ! Standard
      IF(ZN.eq.1.0D+00) go to 120            ! Test rounding underflow
      IF(ZN.gt.1.0D+00) go to 110            ! Test rounding underflow
      IEXP=IEXP-1         ! MUST be < 1
      GO TO 120
  110 IF(ZN.lt.10.0D+00) go to 120           ! Test rounding overflow
      IEXP=IEXP+1         ! MUST be >= 10
      ZN=ZN/10.0D+00
  120 Z = ZN*TENS(IEXP)
c-----------------------------------------------------------------------
C
C     SELECT F OR E FORMAT
C
c-----------------------------------------------------------------------
      IF(Z.LE.ZLOW.OR.Z.GE.ZHIGH) GO TO 150
c-----------------------------------------------------------------------
C
C     F FORMAT
C
C     12345678901
C      X.XXXXXXXX = 9 DIGITS
C      .001234567
C      123456789.
C
c-----------------------------------------------------------------------
C-----DEFINE 6 TO 9 DIGIT MANTISSA WITH ROUNDING
      IPOWER=8-IEXP
      IF(IEXP.LT.0) IPOWER=8
      INMANT=Z*TENS(IPOWER)
C-----CHECK FOR OVERFLOW DUE TO ROUNDING
      if(INMANT.lt.INMAN9) go to 130
      INMANT=INMAN8
      IEXP=IEXP+1
C-----DECIMAL POINT.
  130 IDOT=3+IEXP
      IF(IDOT.LE.2) THEN
C----- IF < 1, MOVE DECIMAL POINT TO COLUMN 2 AND ADD A DIGIT
      IDOT=2
      INMANT=Z*10.0D+00*TENS(IPOWER)
      ENDIF
      FIELD(IDOT)='.'
C-----MANTISSA - LAST DIGIT TO FIRST.
      II=11
      DO 140 I=2,11
      IF(II.EQ.IDOT) GO TO 140
      I2=INMANT/10
      I3=INMANT-10*I2
      FIELD(II)=DIGITS(I3)
      INMANT=I2
  140 II=II-1
      RETURN
c-----------------------------------------------------------------------
C
C     E FORMAT
C
C     12345678901
C      X.XXXXE+NN = 5 DIGITS
C      X.XXXXXE+N = 6 DIGITS
C
C     11/22/2013 - If not negative, use first column
C     12345678901
C     X.XXXXXE+NN = 6 DIGITS
C     X.XXXXXXE+N = 7 DIGITS
c
c-----------------------------------------------------------------------
c
c     Do not cannot use first column
c
C-----DECIMAL POINT IS ALWAYS IN COLUMN 3
  150 FIELD(3)='.'
      KDOT    = 3
      ISTART  = 2
      IF(IABS(IEXP).GE.10) GO TO 160
      ID=8                                 ! 1 Digit Exponent
      INMANT=(1.0D+05)*ZN
      IF(INMANT.lt.INMAN6) go to 170
      INMANT=INMAN5
      IEXP=IEXP+1
      IF(IABS(IEXP).LT.10) GO TO 170
  160 ID=7                                 ! 2 Digit Exponent
      INMANT=(1.0D+04)*ZN
      IF(INMANT.lt.INMAN5) go to 170
      INMANT=INMAN4
      IEXP=IEXP+1
C
C     DEFINE MANTISSA
C
  170 IEXPS=ID+1
      II=ID
      DO 180 I=ISTART,ID
      IF(II.EQ.KDOT) GO TO 180
      I2=INMANT/10
      I3=INMANT-10*I2
      FIELD(II)=DIGITS(I3)
      INMANT=I2
  180 II=II-1
C
C     E
C
      FIELD(IEXPS) = 'E'
      IEXPS = IEXPS + 1
C
C     SIGN OF EXPONENT
C
      IF(IEXP.ge.0) go to 190
      IEXP=-IEXP
      FIELD(IEXPS)='-'
      GO TO 200
  190 FIELD(IEXPS)='+'
C
C     EXPONENT
C
  200 IF(IEXP.lt.10) go to 210
      KEXP=IEXP/10
      FIELD(10)=DIGITS(KEXP)
      IEXP=MOD(IEXP,10)
  210 FIELD(11)=DIGITS(IEXP)
      RETURN
      END
      FUNCTION NXTSEQ(NNSEQ)
C=======================================================================
C
C     DEFINE NEXT SEQUENCE NUMBER FOR ENDF/B OUTPUT. ALLOW FOR
C     MORE THAN 100000 LINES PER EVALUATION BY RESETTING NUMBER
C     TO 1 EVERY TIME 100000 IS REACHED.
C
C=======================================================================
      INCLUDE 'implicit.h'
      NN=NNSEQ+1
      IF(NN.EQ.100000) NN=1
      NXTSEQ=NN
      RETURN
      END
